class RNIB extends Mutator
	config(RNIB);

var TeamGamePlus tgp;
var class SMARTUT_TDM_PRI_CLASS;
var SmartTDMPRI pris[32];
var bool bFirstSpawn;
var bool bEnterHidden;
var bool bMVSpeedUpDone;
var bool bMM;
var float gameStart;
var const string version;
var Pawn dbg;

var bool init;

var config bool bSuperRestore;
var config bool bSpreeEndingBonus;
var config bool bHiddenEntering;
var config bool bPreventPistonCampKills;
var config bool bRelicLimiter;
var config bool bMVSpeedUp;
var config bool bSUT_SB_Forced;

var Pawn lastRoSHolder, lastRoRHolder;

var struct RESTORE_DATA
{
	var string PlayerName;//ID-restore
	var string IP;
	var Pawn p;//ID-running game

//Pawn
	var int RD_DieCount;
	var int RD_KillCount;
//PRI
	var int RD_StartTime;
	var int RD_LastUpdateTime;//used to optimize startTime
	var int RD_Deaths;
	var int RD_Frags;
//SUTPRI/STDMPRI
	var int RD_Teamkills;
	var int RD_AmpKills;
	var int RD_Invisibilities;
	var int RD_Score;
	var int RD_Kills;
	var int RD_Suicides;
	var int RD_HeadShots;
	var int RD_ShieldBelts;
	var int RD_Amplifiers;
} playersArchive[100];
var int archiveLength;

var struct RelicInfo
{
	var int count;
	var bool bBlocked;
	var Pawn p;
} ri[32];

/////////////////////////
function PreBeginPlay()
{
	Log(">> RNIB started version: " $ version);
	
	tgp = TeamGamePlus(Level.Game);
	if(tgp == None)
	{
		Log("NO TDM > RNIB shutting down");
		Self.Destroy();
		return;
	}
}

function ModifyPlayer(Pawn Other)
{
	local int i;
	
	if ( NextMutator != None )
		NextMutator.ModifyPlayer(Other);
		
	if(Other.DieCount == 0)
	{
		//human players start hidden
		if(bHiddenEntering && bFirstSpawn && !Other.IsInState('Dying') && 
			(bEnterHidden || (Level.TimeSeconds > gameStart)) && Other.isA('TournamentPlayer'))
		{
			bEnterHidden = True;
			
			Other.bHidden = True;
			Other.SetCollision(False, False, False);
			Other.GotoState('Dying');//hidden when just entering the game -> no lame frag
		}
				
		//RESTORE/INIT
		if(bSuperRestore)
			RestoreDataFor(Other);
			
			
			if(bSUT_SB_Forced && Other.PlayerReplicationInfo != None && 
				!Other.PlayerReplicationInfo.bWaitingPlayer)
			{
				for(i = 0;i<ArrayCount(pris);i++)
				{
					if(pris[i] != None && pris[i].Owner != None && pris[i].owner.owner == Other)
					{
						pris[i].ToggleStats();
						PlayerPawn(Other).bShowScores = False;
					}
				}
			}
	}
	
	
	//wait for the first spawn -> +1 second -> then hidden entering is enabled
	if(!bFirstSpawn && tgp.bStartMatch)
	{
		bFirstSpawn = True;
		gameStart = Level.TimeSeconds + 1.2;
		//BroadcastMessage("GAME STARTS");//@test
		if(bRelicLimiter)
			SetTimer(1 * Level.TimeDilation, True);//loop for relic limiter
	}
}

function Timer()
{
	local Pawn p;
	local int index;
	local Inventory inv;
	local bool holding;
	
	for(p = Level.PawnList; p != None; p = p.NextPawn)
	{
		if(p.bIsPlayer && p.Visibility > 0)
		{
			index = GetIndex(p);
			
			if(p == lastRoRHolder)
			{
				inv = p.FindInventoryType(class'RelicRegenInventory');
				if(inv != None)
					holding = True;
				else 
				{
					holding = False;
					lastRoRHolder = None;
				}
			}
			else if(p == lastRoSHolder)
			{
				inv = p.FindInventoryType(class'RelicSpeedInventory');
				if(inv != None)
					holding = True;
				else
				{
					holding = False;
					lastRoSHolder = None;
				}
			}
			else
				holding = False;
			
			 
			if(holding)
			{
				if(++ri[index].count > 59)
				{
					ri[index].bBlocked = True;
					p.ClientMessage("[The next 60 seconds you can't take relic of speed/regen.]", ,True);
					
					//remove relic
					inv.Destroy();
				}
			}
			else if(--ri[index].count < 1)
			{
				if(ri[index].bBlocked)
					p.ClientMessage("[You can take all relics again.]");
			
				ri[index].bBlocked = False;
				ri[index].count = 0;
			}
		}
	}
}

function ScoreKill(Pawn Killer, Pawn Other)
{
	if ( NextMutator != None )
		NextMutator.ScoreKill(Killer, Other);

	//a true kill
	if(Killer != None && Other != Killer)
	{
		//bonus frag(s) for ending a killing spree
		if(bSpreeEndingBonus && Other.Spree > 4)
		{
			//Log("spree ended " $ Other.PlayerReplicationInfo.PlayerName);
			if(Other.Spree < 10)
			{
				Killer.PlayerReplicationInfo.Score += 1;
				Killer.ClientMessage("+1 frag for ending a killing spree",, True);
			}
			else
			{
				Killer.PlayerReplicationInfo.Score += 2;
				Killer.ClientMessage("+2 frags for ending a rampage or more",, True);
			}
		}
		
		if(bSuperRestore)
			BackupDataFor(Killer);
	}
		
	//always backup for the victim
	if(bSuperRestore)
		BackupDataFor(Other);
		
	//no teamscore penalty for suiciding
	if(tgp != None && Other.bIsPlayer && ((Killer == None) || Killer.bIsPlayer))
	{
		if ( (Killer == Other) || (Killer == None) )
			tgp.Teams[Other.PlayerReplicationInfo.Team].Score += 1;
	}
}

function RestoreDataFor(Pawn p)
{
	local int index;
	local SmartTDMPRI spri;
	local int found;
	
	//get (new) index in the archive - by last recorded name vs current name + IP for humans
	index = FindInArchive(p.PlayerReplicationInfo.PlayerName, p, found);
	
	if(found == 0)//not found -> no restore | new entry in archive was already generated
		return;
			
	spri = getSTDMPRI(p.PlayerReplicationInfo);//see if spri is there
	
	//can restore now
	//p.PlayerReplicationInfo.PlayerName = playersArchive[index].PlayerName;
	p.DieCount = playersArchive[index].RD_DieCount;
	p.KillCount = playersArchive[index].RD_KillCount;
	//cut out the the time from the last backup-action until now
	p.PlayerReplicationInfo.StartTime = playersArchive[index].RD_StartTime + (Level.TimeSeconds - playersArchive[index].RD_LastUpdateTime);
	p.PlayerReplicationInfo.Deaths = playersArchive[index].RD_Deaths;
	p.PlayerReplicationInfo.Score = playersArchive[index].RD_Frags;
	//Log("saved frags: " $ playersArchive[index].RD_Frags);
	if(spri != None)
	{
		spri.TeamKills = playersArchive[index].RD_TeamKills;
		spri.AmpKills = playersArchive[index].RD_AmpKills;
		spri.Invisibilities = playersArchive[index].RD_Invisibilities;
		spri.Score = playersArchive[index].RD_Score;
		spri.Kills = playersArchive[index].RD_Kills;
		spri.Suicides = playersArchive[index].RD_Suicides;
		spri.HeadShots = playersArchive[index].RD_HeadShots;
		spri.ShieldBelts = playersArchive[index].RD_ShieldBelts;
		spri.Amplifiers = playersArchive[index].RD_Amplifiers;
	}
	
}

//by name + IP
function int FindInArchive(string p_name, Pawn _p, out int found)
{
	local int i;
	local PlayerPawn pp;
	local string ip;
	
	pp = PlayerPawn(_p);
	if(pp != None)
		ip = pp.GetPlayerNetworkAddress();
	
	for(i = 0;i < 100;i++)
	{
		if(p_name == playersArchive[i].PlayerName && 
			(pp == None || InStr(ip, playersArchive[i].IP) != -1))//name && (bot || ip match)
		{
			found = 1;
			playersArchive[i].p = _p;//new reference
			//Log("found " $ p_name $ " old data @ " $ i);
			return i;
		}
	}
	
	found = 0;
	if(archiveLength < 100)
	{
		//not found -> init
		playersArchive[archiveLength].p = _p;
		playersArchive[archiveLength].PlayerName = p_name;
		
		if(pp != None)//human
		{
			//save ip
			playersArchive[archiveLength].IP = pp.GetPlayerNetworkAddress();
			playersArchive[archiveLength].IP = Left(playersArchive[archiveLength].IP, InStr(playersArchive[archiveLength].IP, ":"));
		//	Log("new entry " $ playersArchive[archiveLength].PlayerName $ " / " $ playersArchive[archiveLength].IP);
		}
		//else //bot
			//Log("new entry (BOT)" $ playersArchive[archiveLength].PlayerName);
					
//		Log("init new player " $ p_name $ " @ " $ archiveLength);
		return archiveLength++;;
	}
	else
	{
		Log("Capacity of playersArchive reached!!");
		return -1;
	}	
}

//BackupDataFor - as changes happen, make a backup of potentially changed values, so they are up to date if a restore is needed
function BackupDataFor(Pawn p, optional bool bPickup)
{
	local int index;
	local SmartTDMPRI spri;
	
	index = FindCurrentPlayer(p);
	if(index == -1)
	{
		Log("did not find current player "$p.PlayerReplicationInfo.PlayerName);
		return;
	}
	
	spri = getSTDMPRI(p.PlayerReplicationInfo);
	
	//save data
	playersArchive[index].PlayerName = p.PlayerReplicationInfo.PlayerName;//name may have changed
	if(!bPickup)
	{
		playersArchive[index].RD_DieCount = p.DieCount;
		playersArchive[index].RD_KillCount = p.KillCount;
		playersArchive[index].RD_Frags = p.PlayerReplicationInfo.Score;
		playersArchive[index].RD_StartTime = p.PlayerReplicationInfo.StartTime;//one-time backup? flag
		playersArchive[index].RD_Deaths = p.PlayerReplicationInfo.Deaths;
	}
	
	if(spri != None)
	{
		playersArchive[index].RD_Score = spri.Score;
		
		if(!bPickup)
		{
			playersArchive[index].RD_TeamKills = spri.TeamKills;
			playersArchive[index].RD_AmpKills = spri.AmpKills;
			playersArchive[index].RD_Kills = spri.Kills;
			playersArchive[index].RD_Suicides = spri.Suicides;
			playersArchive[index].RD_HeadShots = spri.HeadShots;
		}
		else
		{
			playersArchive[index].RD_Invisibilities = spri.Invisibilities;
			playersArchive[index].RD_ShieldBelts = spri.ShieldBelts;
			playersArchive[index].RD_Amplifiers = spri.Amplifiers;
		}
	}
		
	//timestamp of last backup
	playersArchive[index].RD_LastUpdateTime = Level.TimeSeconds;
}

function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
	local int i;
	
	if(Other.class == SMARTUT_TDM_PRI_CLASS)
	{
		
		//Log("---NEW STDMPRI---owner "$Other.owner);
		//search empty entry
		for(i = 0;i < 32;i++)
		{
			if(pris[i] == None)
			{
				pris[i] = SmartTDMPRI(Other);
				break;
			}
		}		
	}
	return true;
}

function SmartTDMPRI getSTDMPRI(PlayerReplicationInfo ownerPRI)
{
	local int i;
		
	//Log("search STDMPRI for owner " $ ownerPRI);
	for(i = 31;i > -1;i--)
	{
		if(pris[i] != None && pris[i].owner == ownerPRI)
			return pris[i];
	}
	return None;
}



function bool HandleEndGame()
{
	if(bMVSpeedUp)
	{
		if(dbg != None)
			dbg.ClientMessage(Level.TimeSeconds $ " NextSwitchCountDown = " $ Level.NextSwitchCountdown);
		else
			Log(Level.TimeSeconds $ " NextSwitchCountDown = " $ Level.NextSwitchCountdown);
		
		//wait for map being voted
		if(!bMM)
		{
			bMM = True;
			Level.Game.RegisterMessageMutator(Self);
		}
	}
	
	return Super.HandleEndGame();
}

function bool MutatorBroadcastMessage( Actor Sender, Pawn Receiver, out coerce string Msg, optional bool bBeep, out optional name Type )
{
	//players chose a map or a random one was selected -> game did end before (mid-game voting is ignored)
	if(Type == 'Event' && (Right(Msg, 9) == " has won.")) 
	{
		if(dbg != None)
			dbg.ClientMessage(Level.TimeSeconds $ " (2) NextSwitchCountDown = " $ Level.NextSwitchCountdown);
		else
			Log(Level.TimeSeconds $ " (2) NextSwitchCountDown = " $ Level.NextSwitchCountdown);
			
		if(!bMVSpeedUpDone)
		{
			Level.NextSwitchCountdown = 0;
			bMVSpeedUpDone = True;
		}
	}

	return Super.MutatorBroadcastMessage(Sender, Receiver, Msg, bBeep, Type);
}

function bool HandlePickupQuery(Pawn Other, Inventory item, out byte bAllowPickup)
{

if(bSuperRestore)
	BackupDataFor(Other, True);
	
	if(bRelicLimiter)
	{
	
		if(item.class == class'RelicRegenInventory')
		{
			if(ri[GetIndex(Other)].bBlocked)
			{
				Super.HandlePickupQuery(Other, item, bAllowPickup);
				return True;
			}
			else
			{
				lastRoRHolder = Other;
			}
		}
		else if(item.class == class'RelicSpeedInventory')
		{
			if(ri[GetIndex(Other)].bBlocked)
			{
				Super.HandlePickupQuery(Other, item, bAllowPickup);
				return True;
			}
			else
			{
				lastRoSHolder = Other;
			}
		}
	
		
	}

	if ( NextMutator != None )
		return NextMutator.HandlePickupQuery(Other, item, bAllowPickup);
	return false;
}

function bool PreventDeath(Pawn Killed, Pawn Killer, name damageType, vector HitLocation)
{
	//prevent but pass on the call
	if(bPreventPistonCampKills && Killed.bJustTeleported && Killer.Weapon != None && Killer.Weapon.class == class'ImpactHammer')
	{
		//Log("prevent piston-tele kill ");
		Super.PreventDeath(Killed, Killer, damageType, HitLocation);
		return True;
	}

	if ( NextMutator != None )
		return NextMutator.PreventDeath(Killed,Killer, damageType,HitLocation);
	return false;
}


//find in list
function int FindCurrentPlayer(Pawn _p)
{
	local int i;
		
	for(i = 0;i < 100;i++)
	{
		if(playersArchive[i].p != None && _p == playersArchive[i].p)//found
			return i;
	}

	return -1;
}

function int GetIndex(Pawn _p)
{
	local int i, e;
	
	e = -1;
	for(i = 0;i<32;i++)
	{
		if(_p == ri[i].p)
			return i;
		else if(ri[i].p == None && e == -1)//remember empty slot
			e = i;
			
	}
	
	//new 
	ri[e].p = _p;
	ri[e].bBlocked = False;
	ri[e].count = 0;
	return e;
}


function Mutate(string MutateString, PlayerPawn Sender)
{
	if ( NextMutator != None )
		NextMutator.Mutate(MutateString, Sender);
		
	if(MutateString ~= "RNIB")
		Sender.ClientMessage("RNIB v" $ version);
}


defaultproperties
{
	SMARTUT_TDM_PRI_CLASS=class'SmartUT01.SmartTDMPRI'
	bSuperRestore=True
	bSpreeEndingBonus=True
	bHiddenEntering=True
	bPreventPistonCampKills=True
	bRelicLimiter=True
	bMVSpeedUp=True
	bSUT_SB_Forced=True
	version="3.4.5"
}